package net.sourceforge.squirrel_sql.plugins.mssql.tab;
/*
 * Copyright (C) 2006 Rob Manning
 * manningr@users.sourceforge.net
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import net.sourceforge.squirrel_sql.client.session.ISession;
import net.sourceforge.squirrel_sql.client.session.mainpanel.objecttree.tabs.BasePreparedStatementTab;
import net.sourceforge.squirrel_sql.fw.sql.IDatabaseObjectInfo;
import net.sourceforge.squirrel_sql.fw.util.StringManager;
import net.sourceforge.squirrel_sql.fw.util.StringManagerFactory;
import net.sourceforge.squirrel_sql.fw.util.log.ILogger;
import net.sourceforge.squirrel_sql.fw.util.log.LoggerController;

import java.sql.PreparedStatement;
import java.sql.SQLException;
/**
 * This class will display the details for a MS SQL Server trigger.
 *
 * @author damluar
 */
public class TriggerDetailsTab extends BasePreparedStatementTab
{
	private static final StringManager s_stringMgr =
		StringManagerFactory.getStringManager(TriggerDetailsTab.class);


	/**
	 * This interface defines locale specific strings. This should be
	 * replaced with a property file.
	 */
	private interface i18n
	{
		// i18n[TriggerDetailsTab.title=Details]
		String TITLE = s_stringMgr.getString("TriggerDetailsTab.title");
		// i18n[TriggerDetailsTab.hint=Display trigger details]
		String HINT = s_stringMgr.getString("TriggerDetailsTab.hint");
	}

	/** SQL that retrieves the data. */
	private static String SQL =
        "SELECT     tr.name AS [trigger], " +
        "           tb.name AS [table], " +
        "           sc.name AS [schema], " +
        "           tr.create_date AS [create date], " +
        "           tr.type_desc AS [type], " +
        "           CASE (select type_desc from sys.trigger_events WHERE sys.trigger_events.object_id = tr.object_id AND type = 1) " +
        "               WHEN 'INSERT' THEN 'Yes' ELSE 'No' " +
        "           END [insert event], " +
        "           CASE (select type_desc from sys.trigger_events WHERE sys.trigger_events.object_id = tr.object_id AND type = 2) " +
        "               WHEN 'UPDATE' THEN 'Yes' ELSE 'No' " +
        "           END [update event], " +
        "           CASE (select type_desc from sys.trigger_events WHERE sys.trigger_events.object_id = tr.object_id AND type = 3) " +
        "               WHEN 'DELETE' THEN 'Yes' ELSE 'No' " +
        "           END [delete event] " +
        "FROM       sys.schemas sc JOIN " +
        "           sys.tables tb ON sc.schema_id = tb.schema_id JOIN " +
        "           sys.triggers tr ON tb.object_id = tr.parent_id "  +
        "WHERE tr.name = ? " +
        "AND sc.name = ?";

	/** Logger for this class. */
	private final static ILogger s_log =
		LoggerController.createLogger(TriggerDetailsTab.class);

	public TriggerDetailsTab()
	{
		super(i18n.TITLE, i18n.HINT, true);
	}

	protected PreparedStatement createStatement() throws SQLException
	{
		ISession session = getSession();
        IDatabaseObjectInfo doi = getDatabaseObjectInfo();
        if (s_log.isDebugEnabled()) {
            s_log.debug("Trigger details SQL: "+SQL);
            s_log.debug("Trigger name: "+doi.getSimpleName());
            s_log.debug("Trigger schema: "+doi.getSchemaName());
        }
		PreparedStatement pstmt = session.getSQLConnection().prepareStatement(SQL);
		pstmt.setString(1, doi.getSimpleName());
        pstmt.setString(2, doi.getSchemaName());
		return pstmt;
	}
}
